-- *****************************************************************************
-- *  Copyright (c) 2014 by Michael Fischer (www.emb4fun.de) 
-- *  All rights reserved.
-- *
-- *  This code based on the Verilog example from fpga4fun.com. 
-- *  (http://www.fpga4fun.com/CrossClockDomain2.html)
-- *
-- *  Redistribution and use in source and binary forms, with or without 
-- *  modification, are permitted provided that the following conditions 
-- *  are met:
-- *  
-- *  1. Redistributions of source code must retain the above copyright 
-- *     notice, this list of conditions and the following disclaimer.
-- *  2. Redistributions in binary form must reproduce the above copyright
-- *     notice, this list of conditions and the following disclaimer in the 
-- *     documentation and/or other materials provided with the distribution.
-- *  3. Neither the name of the author nor the names of its contributors may 
-- *     be used to endorse or promote products derived from this software 
-- *     without specific prior written permission.
-- *
-- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
-- *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
-- *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
-- *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
-- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
-- *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
-- *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
-- *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
-- *  SUCH DAMAGE.
-- *
-- *****************************************************************************
-- *  History:
-- *
-- *  14.09.2014  mifi  First version
-- *****************************************************************************

-- *****************************************************************************
-- *  DEFINE: Library                                                          *
-- *****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.jtag_pkg.ALL;


-- *****************************************************************************
-- *  DEFINE: Entity                                                           *
-- *****************************************************************************

entity cdc_flag is
   port ( 
      clkA_i : in  std_logic := '0';
      flag_i : in  std_logic := '0';
      clkB_i : in  std_logic := '0';
      flag_o : out std_logic
   );
end entity cdc_flag;


-- *****************************************************************************
-- *  DEFINE: Architecture                                                     *
-- *****************************************************************************

architecture syn of cdc_flag is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------
   
   signal FlagIn_clkA     : std_logic := '0';
   signal FlagToggle_clkA : std_logic := '0';
   signal SyncA_clkB      : std_logic_vector(2 downto 0) := (others => '0'); 
   signal FlagOut_clkB    : std_logic := '0';
   

begin

   --
   -- As mention above, the Verilog example can be find here:
   -- http://www.fpga4fun.com/CrossClockDomain2.html
   --
   
   --
   -- The trick is to transform the flags into level changes,
   -- and then use the two flip-flops technique.
   --

   FlagIn_clkA <= flag_i;
   
   -- This changes level when the FlagIn_clkA is seen in clkA
   process (clkA_i)
   begin
      if rising_edge(clkA_i) then
         FlagToggle_clkA <= FlagToggle_clkA XOR FlagIn_clkA;
      end if;
   end process; 

   -- Which can then be sync-ed to clkB
   process (clkB_i)
   begin
      if rising_edge(clkB_i) then
         SyncA_clkB <= SyncA_clkB(1 downto 0) & FlagToggle_clkA;
      end if;
   end process; 

   -- And recreate the flag in clkB
   FlagOut_clkB <= SyncA_clkB(2) XOR SyncA_clkB(1);

   
   --------------------------------------------------------
   -- Output
   --------------------------------------------------------
   
   flag_o <= FlagOut_clkB;
   
end architecture syn;

-- *** EOF ***

